@@ -30,14 +30,14 @@
 /*** ChannelWidget ***/
 
 ChannelWidget::ChannelWidget(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& x) :
-    Gtk::EventBox(cobject),
+    Gtk::Widget(cobject),
     can_decibel(false),
     volumeScaleEnabled(true),
     last(false) {
 
-    x->get_widget("channelLabel", channelLabel);
-    x->get_widget("volumeLabel", volumeLabel);
-    x->get_widget("volumeScale", volumeScale);
+    channelLabel = x->get_widget<Gtk::Label>("channelLabel");
+    volumeLabel = x->get_widget<Gtk::Label>("volumeLabel");
+    volumeScale = x->get_widget<Gtk::Scale>("volumeScale");
 
     volumeScale->set_range((double)PA_VOLUME_MUTED, (double)PA_VOLUME_UI_MAX);
     volumeScale->set_value((double)PA_VOLUME_NORM);
@@ -47,15 +47,47 @@ ChannelWidget::ChannelWidget(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Bu
     volumeScale->signal_value_changed().connect(sigc::mem_fun(*this, &ChannelWidget::onVolumeScaleValueChanged));
 }
 
-ChannelWidget* ChannelWidget::create() {
+ChannelWidget* ChannelWidget::createOne(MinimalStreamWidget *owner, int channelIndex, pa_channel_position channelPosition, bool can_decibel) {
     ChannelWidget* w;
-    Glib::RefPtr<Gtk::Builder> x = Gtk::Builder::create();
-    x->add_from_file(GLADE_FILE, "adjustment1");
-    x->add_from_file(GLADE_FILE, "channelWidget");
-    x->get_widget_derived("channelWidget", w);
+    Glib::RefPtr<Gtk::Builder> x = Gtk::Builder::create_from_resource("/org/pulseaudio/pavucontrol/ui/channelwidget.ui", "channelWidget");
+    w = Gtk::Builder::get_widget_derived<ChannelWidget>(x, "channelWidget");
+    w->reference();
+
+    w->channel = channelIndex;
+    w->can_decibel = can_decibel;
+    w->minimalStreamWidget = owner;
+
+    char text[64];
+    snprintf(text, sizeof(text), "<b>%s</b>", pa_channel_position_to_pretty_string(channelPosition));
+    w->channelLabel->set_markup(text);
+
     return w;
 }
 
+void ChannelWidget::create(MinimalStreamWidget *owner, const pa_channel_map &m, bool can_decibel, ChannelWidget *widgets[PA_CHANNELS_MAX]) {
+    int maxLabelWidth = 0;
+
+    for (int i = 0; i < m.channels; i++) {
+        widgets[i] = ChannelWidget::createOne(owner, i, m.map[i], can_decibel);
+
+        Gtk::Requisition minimumSize;
+        Gtk::Requisition naturalSize;
+        widgets[i]->channelLabel->get_preferred_size(minimumSize, naturalSize);
+        if (naturalSize.get_width() > maxLabelWidth)
+            maxLabelWidth = naturalSize.get_width();
+    }
+
+    widgets[m.channels - 1]->last = true;
+
+    /* The channel labels have different widths by default, which makes the
+     * volume slider widths different too. The volume sliders must have the
+     * same width, otherwise it's very hard to see how the volumes of different
+     * channels relate to each other, so we have to change all channel labels
+     * to have the same width. */
+    for (int i = 0; i < m.channels; i++)
+        widgets[i]->channelLabel->set_size_request(maxLabelWidth, -1);
+}
+
 void ChannelWidget::setVolume(pa_volume_t volume) {
     double v;
     char txt[64];
@@ -64,16 +96,17 @@ void ChannelWidget::setVolume(pa_volume_t volume) {
     if (can_decibel) {
         double dB = pa_sw_volume_to_dB(volume);
         if (dB > PA_DECIBEL_MININFTY)
-            snprintf(txt, sizeof(txt), "<small>%0.0f%% (%0.2fdB)</small>", v, dB);
+            snprintf(txt, sizeof(txt), _("<small>%0.0f%% (%0.2f dB)</small>"), v, dB);
         else
-            snprintf(txt, sizeof(txt), "<small>%0.0f%% (-&#8734;dB)</small>", v);
+            snprintf(txt, sizeof(txt), _("<small>%0.0f%% (-&#8734; dB)</small>"), v);
     }
     else
-        snprintf(txt, sizeof(txt), "%0.0f%%", v);
+        snprintf(txt, sizeof(txt), _("%0.0f%%"), v);
     volumeLabel->set_markup(txt);
 
     volumeScaleEnabled = false;
     volumeScale->set_value(volume > PA_VOLUME_UI_MAX ? PA_VOLUME_UI_MAX : volume);
+    currentVolume = volumeScale->get_value();
     volumeScaleEnabled = true;
 }
 
@@ -86,11 +119,12 @@ void ChannelWidget::onVolumeScaleValueChanged() {
         return;
 
     pa_volume_t volume = (pa_volume_t) volumeScale->get_value();
-    minimalStreamWidget->updateChannelVolume(channel, volume);
+    if (volume != currentVolume)
+        minimalStreamWidget->updateChannelVolume(channel, volume);
 }
 
 void ChannelWidget::set_sensitive(bool enabled) {
-    Gtk::EventBox::set_sensitive(enabled);
+    Gtk::Widget::set_sensitive(enabled);
 
     channelLabel->set_sensitive(enabled);
     volumeLabel->set_sensitive(enabled);
@@ -104,7 +138,7 @@ void ChannelWidget::setBaseVolume(pa_volume_t v) {
     gtk_scale_add_mark(GTK_SCALE(volumeScale->gobj()), (double)PA_VOLUME_MUTED, (GtkPositionType) GTK_POS_BOTTOM,
                        last ? (can_decibel ? _("<small>Silence</small>") : _("<small>Min</small>")) : NULL);
     gtk_scale_add_mark(GTK_SCALE(volumeScale->gobj()), (double)PA_VOLUME_NORM, (GtkPositionType) GTK_POS_BOTTOM,
-                       last ? _("<small>100% (0dB)</small>") : NULL);
+                       last ? _("<small>100% (0 dB)</small>") : NULL);
 
     if (v > PA_VOLUME_MUTED && v < PA_VOLUME_NORM) {
         gtk_scale_add_mark(GTK_SCALE(volumeScale->gobj()), (double)v, (GtkPositionType) GTK_POS_BOTTOM,
