@@ -25,16 +25,20 @@
 
 class MinimalStreamWidget;
 
-class ChannelWidget : public Gtk::EventBox {
+class ChannelWidget : public Gtk::Widget {
 public:
-    ChannelWidget(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& x);
-    static ChannelWidget* create();
+    ChannelWidget(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& x);
+
+    /* This creates multiple ChannelWidgets based on the given channel map. The
+     * widgets are stored in the caller-provided array. */
+    static void create(MinimalStreamWidget *owner, const pa_channel_map &m, bool can_decibel,
+                       ChannelWidget *widgets[PA_CHANNELS_MAX]);
 
     void setVolume(pa_volume_t volume);
 
     Gtk::Label *channelLabel;
     Gtk::Label *volumeLabel;
-    Gtk::HScale *volumeScale;
+    Gtk::Scale *volumeScale;
 
     int channel;
     MinimalStreamWidget *minimalStreamWidget;
@@ -43,11 +47,15 @@ public:
 
     bool can_decibel;
     bool volumeScaleEnabled;
+    bool last;
 
     virtual void set_sensitive(bool enabled);
     virtual void setBaseVolume(pa_volume_t);
-    virtual void setSteps(unsigned n);
+
+private:
+    pa_volume_t currentVolume;
+    static ChannelWidget *createOne(MinimalStreamWidget *owner, int channelIndex, pa_channel_position channelPosition,
+                                    bool can_decibel);
 };
 
-
 #endif
