@@ -21,26 +21,46 @@
 #ifndef minimalstreamwidget_h
 #define minimalstreamwidget_h
 
-#include "pavucontrol.h"
+#include <gtkmm.h>
+#include <pulse/pulseaudio.h>
 
-class MinimalStreamWidget : public Gtk::VBox {
+#define PEAKS_RATE 144
+
+class MinimalStreamWidget : public Gtk::Box {
 public:
-    MinimalStreamWidget(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& x);
+    MinimalStreamWidget(BaseObjectType* cobject);
+    virtual ~MinimalStreamWidget();
 
-    Gtk::VBox *channelsVBox;
+    /* Subclass constructors are expected to initialize these variables.
+     * MinimalStreamWidget can't initialize these, because the glade object
+     * id's depend on the subclass type. */
+    Gtk::Box *channelsVBox;
     Gtk::Label *nameLabel, *boldNameLabel;
     Gtk::Image *iconImage;
+
     Gtk::ProgressBar peakProgressBar;
     double lastPeak;
+    pa_stream *peak;
 
     bool updating;
 
     virtual void onMuteToggleButton() = 0;
+    virtual void onLockToggleButton() = 0;
     virtual void updateChannelVolume(int channel, pa_volume_t v) = 0;
 
     bool volumeMeterEnabled;
     void enableVolumeMeter();
     void updatePeak(double v);
+    void setVolumeMeterVisible(bool v);
+
+protected:
+    /* Subclasses must call this after the constructor to finalize the initial
+     * layout. */
+    void init();
+
+private :
+    bool volumeMeterVisible;
+
 };
 
 #endif
