@@ -23,40 +23,53 @@
 #endif
 
 #include <pulse/pulseaudio.h>
-#include <pulse/glib-mainloop.h>
 #include <pulse/ext-stream-restore.h>
 #include <pulse/ext-device-manager.h>
-
-#include <canberra-gtk.h>
+#ifdef HAVE_PULSE_MESSAGING_API
+#include <json-glib/json-glib.h>
+#endif
 
 #include "pavucontrol.h"
 #include "i18n.h"
-#include "minimalstreamwidget.h"
-#include "channelwidget.h"
 #include "streamwidget.h"
-#include "cardwidget.h"
-#include "sinkwidget.h"
-#include "sourcewidget.h"
 #include "sinkinputwidget.h"
 #include "sourceoutputwidget.h"
-#include "rolewidget.h"
 #include "mainwindow.h"
+#include "pavuapplication.h"
+
+#include <unordered_map>
+#include <utility>
+#include <memory>
+
+using WindowAndCardName = std::pair<MainWindow*, std::string>;
 
 static pa_context* context = NULL;
 static pa_mainloop_api* api = NULL;
 static int n_outstanding = 0;
-static int default_tab = 0;
+static int tab_number = 0;
+static bool retry = false;
 static int reconnect_timeout = 1;
 
-void show_error(const char *txt) {
+void show_error_finish (const Glib::RefPtr<Gio::AsyncResult>& result) {
+    PavuApplication::get_instance().quit();
+}
+
+void show_error(Gtk::Widget* widget, const char *txt) {
+    Gtk::Root *root = widget->get_root();
     char buf[256];
 
     snprintf(buf, sizeof(buf), "%s: %s", txt, pa_strerror(pa_context_errno(context)));
 
-    Gtk::MessageDialog dialog(buf, false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_CLOSE, true);
-    dialog.run();
-
-    Gtk::Main::quit();
+    auto dialog = Gtk::AlertDialog::create(buf);
+    dialog->set_modal(true);
+    if (GTK_IS_WINDOW(root->gobj())) {
+        GtkWindow* w = (GtkWindow*) root->gobj();
+        Gtk::Window* window = Glib::wrap(w);
+        window->present();
+        dialog->choose(*window, sigc::ptr_fun(show_error_finish));
+    } else {
+        dialog->choose(sigc::ptr_fun(show_error_finish));
+    }
 }
 
 static void dec_outstanding(MainWindow *w) {
@@ -64,19 +77,288 @@ static void dec_outstanding(MainWindow *w) {
         return;
 
     if (--n_outstanding <= 0) {
-        w->get_window()->set_cursor();
+        w->set_cursor(Gdk::Cursor::create("default"));;
         w->setConnectionState(true);
     }
 }
 
-void card_cb(pa_context *, const pa_card_info *i, int eol, void *userdata) {
+#ifdef HAVE_PULSE_MESSAGING_API
+
+std::string card_message_handler_path(const std::string& name) {
+    return "/card/" + name;
+}
+
+std::string card_bluez_message_handler_path(const std::string& name) {
+    return "/card/" + name + "/bluez";
+}
+
+static void context_bluetooth_card_codec_list_cb(pa_context *c, int success, char *response, void *userdata) {
+    auto u = std::unique_ptr<WindowAndCardName>(reinterpret_cast<WindowAndCardName*>(userdata));
+
+    if (!success)
+        return;
+
+    int err = 0;
+
+    GError *gerror = NULL;
+    JsonParser *parser = json_parser_new();
+
+    if (!json_parser_load_from_data(parser, response, strlen(response), &gerror)) {
+        g_debug(_("could not read JSON from list-codecs message response: %s"), gerror->message);
+        g_error_free(gerror);
+        g_object_unref(parser);
+        return;
+    }
+
+    JsonNode *root = json_parser_get_root(parser);
+
+    if (!root || JSON_NODE_TYPE(root) != JSON_NODE_ARRAY) {
+        g_debug(_("list-codecs message response is not a JSON array"));
+        g_object_unref(parser);
+        return;
+    }
+
+    JsonArray *array = json_node_get_array(root);
+
+    std::unordered_map<std::string, std::string> codecs;
+
+    for (guint i = 0; i < json_array_get_length(array); ++i) {
+        const char *path;
+        const char *description;
+        JsonNode *v;
+
+        JsonObject *object = json_array_get_object_element(array, i);
+        if (!object) {
+            err = -1;
+            break;
+        }
+
+        v = json_object_get_member(object, "name");
+        if (!v) {
+            err = -1;
+            break;
+        }
+
+        path = json_node_get_string(v);
+        if (!path) {
+            err = -1;
+            break;
+        }
+
+        v = json_object_get_member(object, "description");
+        if (!v) {
+            err = -1;
+            break;
+        }
+
+        description = json_node_get_string(v);
+        if (!description)
+            description = "";
+
+        codecs[path] = description;
+    }
+
+    g_object_unref(parser);
+
+    if (err < 0) {
+        g_debug(_("list-codecs message response could not be parsed correctly"));
+        codecs.clear();
+        return;
+    }
+
+    u->first->updateCardCodecs(u->second, codecs);
+}
+
+static void context_bluetooth_card_active_codec_cb(pa_context *c, int success, char *response, void *userdata) {
+    auto u = std::unique_ptr<WindowAndCardName>(reinterpret_cast<WindowAndCardName*>(userdata));
+
+    if (!success)
+        return;
+
+    const char *name;
+    GError *gerror = NULL;
+
+    JsonParser *parser = json_parser_new();
+
+    if (!json_parser_load_from_data(parser, response, strlen(response), &gerror)) {
+        g_debug(_("could not read JSON from get-codec message response: %s"), gerror->message);
+        g_error_free(gerror);
+        g_object_unref(parser);
+        return;
+    }
+
+    JsonNode *root = json_parser_get_root(parser);
+
+    if (!root || JSON_NODE_TYPE(root) != JSON_NODE_VALUE) {
+        g_debug(_("get-codec message response is not a JSON value"));
+        g_object_unref(parser);
+        return;
+    }
+
+    name = json_node_get_string(root);
+
+    if (!name) {
+        g_debug(_("could not get codec name from get-codec message response"));
+        g_object_unref(parser);
+        return;
+    }
+
+    u->first->setActiveCodec(u->second, name);
+
+    g_object_unref(parser);
+}
+
+static void context_card_profile_is_sticky_cb(pa_context *c, int success, char *response, void *userdata) {
+    auto u = std::unique_ptr<WindowAndCardName>(reinterpret_cast<WindowAndCardName*>(userdata));
+
+    if (!success)
+        return;
+
+    gboolean profile_is_sticky;
+    GError *gerror = NULL;
+
+    JsonParser *parser = json_parser_new();
+
+    if (!json_parser_load_from_data(parser, response, strlen(response), &gerror)) {
+        g_debug(_("could not read JSON from get-profile-sticky message response: %s"), gerror->message);
+        g_error_free(gerror);
+        g_object_unref(parser);
+        return;
+    }
+
+    JsonNode *root = json_parser_get_root(parser);
+
+    if (!root || JSON_NODE_TYPE(root) != JSON_NODE_VALUE) {
+        g_debug(_("get-profile-sticky message response is not a JSON value"));
+        g_object_unref(parser);
+        return;
+    }
+
+    profile_is_sticky = json_node_get_boolean(root);
+
+    u->first->setCardProfileIsSticky(u->second, profile_is_sticky);
+
+    g_object_unref(parser);
+}
+
+template<typename U> void send_message(pa_context *c, const char *target, const char *request, pa_context_string_cb_t cb, const U& u)
+{
+    auto send_message_userdata = new U(u);
+
+    pa_operation *o = pa_context_send_message_to_object(c, target, request, NULL, cb, send_message_userdata);
+
+    if (!o) {
+        delete send_message_userdata;
+        g_debug(_("pa_context_send_message_to_object() failed: %s"), pa_strerror(pa_context_errno(context)));
+        return;
+    }
+    pa_operation_unref(o);
+}
+
+static void context_message_handlers_cb(pa_context *c, int success, char *response, void *userdata) {
+    auto u = std::unique_ptr<WindowAndCardName>(reinterpret_cast<WindowAndCardName*>(userdata));
+
+    if (!success)
+        return;
+
+    int err = 0;
+
+    GError *gerror = NULL;
+    JsonParser *parser = json_parser_new();
+
+    if (!json_parser_load_from_data(parser, response, strlen(response), &gerror)) {
+        g_debug(_("could not read JSON from list-handlers message response: %s"), gerror->message);
+        g_error_free(gerror);
+        g_object_unref(parser);
+        return;
+    }
+
+    JsonNode *root = json_parser_get_root(parser);
+
+    if (!root || JSON_NODE_TYPE(root) != JSON_NODE_ARRAY) {
+        g_debug(_("list-handlers message response is not a JSON array"));
+        g_object_unref(parser);
+        return;
+    }
+
+    JsonArray *array = json_node_get_array(root);
+
+    std::unordered_map<std::string, std::string> message_handler_map;
+
+    for (guint i = 0; i < json_array_get_length(array); ++i) {
+        const char *path;
+        const char *description;
+        JsonNode *v;
+
+        JsonObject *object = json_array_get_object_element(array, i);
+        if (!object) {
+            err = -1;
+            break;
+        }
+
+        v = json_object_get_member(object, "name");
+        if (!v) {
+            err = -1;
+            break;
+        }
+
+        path = json_node_get_string(v);
+        if (!path) {
+            err = -1;
+            break;
+        }
+
+        v = json_object_get_member(object, "description");
+        if (!v) {
+            err = -1;
+            break;
+        }
+
+        description = json_node_get_string(v);
+        if (!description)
+            description = "";
+
+        message_handler_map[path] = description;
+    }
+
+    g_object_unref(parser);
+
+    if (err < 0) {
+        g_debug(_("list-handlers message response could not be parsed correctly"));
+        message_handler_map.clear();
+        return;
+    }
+
+    /* only send requests if card bluez message handler is registered */
+    auto e = message_handler_map.find(card_bluez_message_handler_path(u->second));
+
+    if (e != message_handler_map.end()) {
+
+        /* get-codec: retrieve active codec name */
+        send_message(c, e->first.c_str(), "get-codec", context_bluetooth_card_active_codec_cb, *u);
+
+        /* list-codecs: retrieve list of codecs */
+        send_message(c, e->first.c_str(), "list-codecs", context_bluetooth_card_codec_list_cb, *u);
+    }
+
+    /* send requests to card if card message handler is registered */
+    e = message_handler_map.find(card_message_handler_path(u->second));
+
+    if (e != message_handler_map.end()) {
+        /* get-profile-sticky: retrieve active codec name */
+        send_message(c, e->first.c_str(), "get-profile-sticky", context_card_profile_is_sticky_cb, *u);
+    }
+}
+#endif
+
+void card_cb(pa_context *c, const pa_card_info *i, int eol, void *userdata) {
     MainWindow *w = static_cast<MainWindow*>(userdata);
 
     if (eol < 0) {
         if (pa_context_errno(context) == PA_ERR_NOENTITY)
             return;
 
-        show_error(_("Card callback failure"));
+        show_error(w, _("Card callback failure"));
         return;
     }
 
@@ -86,6 +368,11 @@ void card_cb(pa_context *, const pa_card_info *i, int eol, void *userdata) {
     }
 
     w->updateCard(*i);
+
+#ifdef HAVE_PULSE_MESSAGING_API
+    /* initiate requesting bluetooth codec list */
+    send_message(c, "/core", "list-handlers", context_message_handlers_cb, WindowAndCardName(w, i->name));
+#endif
 }
 
 #if HAVE_EXT_DEVICE_RESTORE_API
@@ -99,7 +386,7 @@ void sink_cb(pa_context *c, const pa_sink_info *i, int eol, void *userdata) {
         if (pa_context_errno(context) == PA_ERR_NOENTITY)
             return;
 
-        show_error(_("Sink callback failure"));
+        show_error(w, _("Sink callback failure"));
         return;
     }
 
@@ -123,7 +410,7 @@ void source_cb(pa_context *, const pa_source_info *i, int eol, void *userdata) {
         if (pa_context_errno(context) == PA_ERR_NOENTITY)
             return;
 
-        show_error(_("Source callback failure"));
+        show_error(w, _("Source callback failure"));
         return;
     }
 
@@ -142,7 +429,7 @@ void sink_input_cb(pa_context *, const pa_sink_input_info *i, int eol, void *use
         if (pa_context_errno(context) == PA_ERR_NOENTITY)
             return;
 
-        show_error(_("Sink input callback failure"));
+        show_error(w, _("Sink input callback failure"));
         return;
     }
 
@@ -161,7 +448,7 @@ void source_output_cb(pa_context *, const pa_source_output_info *i, int eol, voi
         if (pa_context_errno(context) == PA_ERR_NOENTITY)
             return;
 
-        show_error(_("Source output callback failure"));
+        show_error(w, _("Source output callback failure"));
         return;
     }
 
@@ -170,20 +457,10 @@ void source_output_cb(pa_context *, const pa_source_output_info *i, int eol, voi
         if (n_outstanding > 0) {
             /* At this point all notebook pages have been populated, so
              * let's open one that isn't empty */
-            if (default_tab != -1) {
-                if (default_tab < 1 || default_tab > w->notebook->get_n_pages()) {
-                    if (w->sinkInputWidgets.size() > 0)
-                        w->notebook->set_current_page(0);
-                    else if (w->sourceOutputWidgets.size() > 0)
-                        w->notebook->set_current_page(1);
-                    else if (w->sourceWidgets.size() > 0 && w->sinkWidgets.size() == 0)
-                        w->notebook->set_current_page(3);
-                    else
-                        w->notebook->set_current_page(2);
-                } else {
-                    w->notebook->set_current_page(default_tab - 1);
-                }
-                default_tab = -1;
+            if (tab_number != 0) {
+                w->selectTab(tab_number);
+            } else {
+                w->selectBestTab();
             }
         }
 
@@ -201,7 +478,7 @@ void client_cb(pa_context *, const pa_client_info *i, int eol, void *userdata) {
         if (pa_context_errno(context) == PA_ERR_NOENTITY)
             return;
 
-        show_error(_("Client callback failure"));
+        show_error(w, _("Client callback failure"));
         return;
     }
 
@@ -217,7 +494,7 @@ void server_info_cb(pa_context *, const pa_server_info *i, void *userdata) {
     MainWindow *w = static_cast<MainWindow*>(userdata);
 
     if (!i) {
-        show_error(_("Server info callback failure"));
+        show_error(w, _("Server info callback failure"));
         return;
     }
 
@@ -253,7 +530,7 @@ static void ext_stream_restore_subscribe_cb(pa_context *c, void *userdata) {
     pa_operation *o;
 
     if (!(o = pa_ext_stream_restore_read(c, ext_stream_restore_read_cb, w))) {
-        show_error(_("pa_ext_stream_restore_read() failed"));
+        show_error(w, _("pa_ext_stream_restore_read() failed"));
         return;
     }
 
@@ -292,7 +569,7 @@ static void ext_device_restore_subscribe_cb(pa_context *c, pa_device_type_t type
         return;
 
     if (!(o = pa_ext_device_restore_read_formats(c, type, idx, ext_device_restore_read_cb, w))) {
-        show_error(_("pa_ext_device_restore_read_sink_formats() failed"));
+        show_error(w, _("pa_ext_device_restore_read_sink_formats() failed"));
         return;
     }
 
@@ -329,7 +606,7 @@ static void ext_device_manager_subscribe_cb(pa_context *c, void *userdata) {
     pa_operation *o;
 
     if (!(o = pa_ext_device_manager_read(c, ext_device_manager_read_cb, w))) {
-        show_error(_("pa_ext_device_manager_read() failed"));
+        show_error(w, _("pa_ext_device_manager_read() failed"));
         return;
     }
 
@@ -346,7 +623,7 @@ void subscribe_cb(pa_context *c, pa_subscription_event_type_t t, uint32_t index,
             else {
                 pa_operation *o;
                 if (!(o = pa_context_get_sink_info_by_index(c, index, sink_cb, w))) {
-                    show_error(_("pa_context_get_sink_info_by_index() failed"));
+                    show_error(w, _("pa_context_get_sink_info_by_index() failed"));
                     return;
                 }
                 pa_operation_unref(o);
@@ -359,7 +636,7 @@ void subscribe_cb(pa_context *c, pa_subscription_event_type_t t, uint32_t index,
             else {
                 pa_operation *o;
                 if (!(o = pa_context_get_source_info_by_index(c, index, source_cb, w))) {
-                    show_error(_("pa_context_get_source_info_by_index() failed"));
+                    show_error(w, _("pa_context_get_source_info_by_index() failed"));
                     return;
                 }
                 pa_operation_unref(o);
@@ -372,7 +649,7 @@ void subscribe_cb(pa_context *c, pa_subscription_event_type_t t, uint32_t index,
             else {
                 pa_operation *o;
                 if (!(o = pa_context_get_sink_input_info(c, index, sink_input_cb, w))) {
-                    show_error(_("pa_context_get_sink_input_info() failed"));
+                    show_error(w, _("pa_context_get_sink_input_info() failed"));
                     return;
                 }
                 pa_operation_unref(o);
@@ -385,7 +662,7 @@ void subscribe_cb(pa_context *c, pa_subscription_event_type_t t, uint32_t index,
             else {
                 pa_operation *o;
                 if (!(o = pa_context_get_source_output_info(c, index, source_output_cb, w))) {
-                    show_error(_("pa_context_get_sink_input_info() failed"));
+                    show_error(w, _("pa_context_get_sink_input_info() failed"));
                     return;
                 }
                 pa_operation_unref(o);
@@ -398,7 +675,7 @@ void subscribe_cb(pa_context *c, pa_subscription_event_type_t t, uint32_t index,
             else {
                 pa_operation *o;
                 if (!(o = pa_context_get_client_info(c, index, client_cb, w))) {
-                    show_error(_("pa_context_get_client_info() failed"));
+                    show_error(w, _("pa_context_get_client_info() failed"));
                     return;
                 }
                 pa_operation_unref(o);
@@ -408,7 +685,7 @@ void subscribe_cb(pa_context *c, pa_subscription_event_type_t t, uint32_t index,
         case PA_SUBSCRIPTION_EVENT_SERVER: {
                 pa_operation *o;
                 if (!(o = pa_context_get_server_info(c, server_info_cb, w))) {
-                    show_error(_("pa_context_get_server_info() failed"));
+                    show_error(w, _("pa_context_get_server_info() failed"));
                     return;
                 }
                 pa_operation_unref(o);
@@ -421,7 +698,7 @@ void subscribe_cb(pa_context *c, pa_subscription_event_type_t t, uint32_t index,
             else {
                 pa_operation *o;
                 if (!(o = pa_context_get_card_info_by_index(c, index, card_cb, w))) {
-                    show_error(_("pa_context_get_card_info_by_index() failed"));
+                    show_error(w, _("pa_context_get_card_info_by_index() failed"));
                     return;
                 }
                 pa_operation_unref(o);
@@ -464,7 +741,7 @@ void context_state_callback(pa_context *c, void *userdata) {
                                             PA_SUBSCRIPTION_MASK_CLIENT|
                                             PA_SUBSCRIPTION_MASK_SERVER|
                                             PA_SUBSCRIPTION_MASK_CARD), NULL, NULL))) {
-                show_error(_("pa_context_subscribe() failed"));
+                show_error(w, _("pa_context_subscribe() failed"));
                 return;
             }
             pa_operation_unref(o);
@@ -473,49 +750,49 @@ void context_state_callback(pa_context *c, void *userdata) {
             n_outstanding = 0;
 
             if (!(o = pa_context_get_server_info(c, server_info_cb, w))) {
-                show_error(_("pa_context_get_server_info() failed"));
+                show_error(w, _("pa_context_get_server_info() failed"));
                 return;
             }
             pa_operation_unref(o);
             n_outstanding++;
 
             if (!(o = pa_context_get_client_info_list(c, client_cb, w))) {
-                show_error(_("pa_context_client_info_list() failed"));
+                show_error(w, _("pa_context_client_info_list() failed"));
                 return;
             }
             pa_operation_unref(o);
             n_outstanding++;
 
             if (!(o = pa_context_get_card_info_list(c, card_cb, w))) {
-                show_error(_("pa_context_get_card_info_list() failed"));
+                show_error(w, _("pa_context_get_card_info_list() failed"));
                 return;
             }
             pa_operation_unref(o);
             n_outstanding++;
 
             if (!(o = pa_context_get_sink_info_list(c, sink_cb, w))) {
-                show_error(_("pa_context_get_sink_info_list() failed"));
+                show_error(w, _("pa_context_get_sink_info_list() failed"));
                 return;
             }
             pa_operation_unref(o);
             n_outstanding++;
 
             if (!(o = pa_context_get_source_info_list(c, source_cb, w))) {
-                show_error(_("pa_context_get_source_info_list() failed"));
+                show_error(w, _("pa_context_get_source_info_list() failed"));
                 return;
             }
             pa_operation_unref(o);
             n_outstanding++;
 
             if (!(o = pa_context_get_sink_input_info_list(c, sink_input_cb, w))) {
-                show_error(_("pa_context_get_sink_input_info_list() failed"));
+                show_error(w, _("pa_context_get_sink_input_info_list() failed"));
                 return;
             }
             pa_operation_unref(o);
             n_outstanding++;
 
             if (!(o = pa_context_get_source_output_info_list(c, source_output_cb, w))) {
-                show_error(_("pa_context_get_source_output_info_list() failed"));
+                show_error(w, _("pa_context_get_source_output_info_list() failed"));
                 return;
             }
             pa_operation_unref(o);
@@ -581,7 +858,7 @@ void context_state_callback(pa_context *c, void *userdata) {
 
         case PA_CONTEXT_TERMINATED:
         default:
-            Gtk::Main::quit();
+            w->get_application()->quit();
             return;
     }
 }
@@ -615,72 +892,39 @@ gboolean connect_to_pulse(gpointer userdata) {
             w->setConnectingMessage(_("Connection to PulseAudio failed. Automatic retry in 5s\n\n"
                 "In this case this is likely because PULSE_SERVER in the Environment/X11 Root Window Properties\n"
                 "or default-server in client.conf is misconfigured.\n"
-                "This situation can also arrise when PulseAudio crashed and left stale details in the X11 Root Window.\n"
+                "This situation can also arise when PulseAudio crashed and left stale details in the X11 Root Window.\n"
                 "If this is the case, then PulseAudio should autospawn again, or if this is not configured you should\n"
                 "run start-pulseaudio-x11 manually."));
             reconnect_timeout = 5;
         }
         else {
-            reconnect_timeout = -1;
-            Gtk::Main::quit();
+            if(!retry) {
+                reconnect_timeout = -1;
+                w->get_application()->quit();
+            } else {
+                g_debug(_("Connection failed, attempting reconnect"));
+                reconnect_timeout = 5;
+                g_timeout_add_seconds(reconnect_timeout, connect_to_pulse, w);
+            }
         }
     }
 
     return false;
 }
 
-int main(int argc, char *argv[]) {
+MainWindow* pavucontrol_get_window(pa_glib_mainloop *m, bool maximize, bool _retry, int _tab_number) {
 
-    /* Initialize the i18n stuff */
-    bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
-    bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
-    textdomain(GETTEXT_PACKAGE);
+    MainWindow* mainWindow = NULL;
 
-    signal(SIGPIPE, SIG_IGN);
+    tab_number = _tab_number;
+    retry = _retry;
 
+    mainWindow = MainWindow::create(maximize);
 
-    Glib::OptionContext options;
-    options.set_summary("PulseAudio Volume Control");
-    options.set_help_enabled();
+    api = pa_glib_mainloop_get_api(m);
+    g_assert(api);
 
-    Glib::OptionGroup group("pulseaudio", "PAVUControl");
+    connect_to_pulse(mainWindow);
 
-    Glib::OptionEntry entry;
-    entry.set_long_name("tab");
-    entry.set_short_name('t');
-    entry.set_description(_("Select a specific tab on load."));
-    group.add_entry(entry, default_tab);
-
-    options.set_main_group(group);
-
-    try {
-        Gtk::Main kit(argc, argv, options);
-
-        ca_context_set_driver(ca_gtk_context_get(), "pulse");
-
-        MainWindow* mainWindow = MainWindow::create();
-
-        pa_glib_mainloop *m = pa_glib_mainloop_new(g_main_context_default());
-        g_assert(m);
-        api = pa_glib_mainloop_get_api(m);
-        g_assert(api);
-
-        connect_to_pulse(mainWindow);
-        if (reconnect_timeout >= 0)
-            Gtk::Main::run(*mainWindow);
-
-        if (reconnect_timeout < 0)
-            show_error(_("Fatal Error: Unable to connect to PulseAudio"));
-
-        delete mainWindow;
-
-        if (context)
-            pa_context_unref(context);
-        pa_glib_mainloop_free(m);
-    } catch ( const Glib::OptionError & e ) {
-        fprintf(stderr, "%s", options.get_help().c_str());
-        return 1;
-    }
-
-    return 0;
+    return mainWindow;
 }
