@@ -29,7 +29,7 @@ class MainWindow;
 
 class SinkInputWidget : public StreamWidget {
 public:
-    SinkInputWidget(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& x);
+    SinkInputWidget(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& x);
     static SinkInputWidget* create(MainWindow* mainWindow);
     ~SinkInputWidget(void);
 
@@ -38,36 +38,14 @@ public:
     uint32_t index, clientIndex;
     void setSinkIndex(uint32_t idx);
     uint32_t sinkIndex();
+    void updateDeviceComboBox();
     virtual void executeVolumeUpdate();
-    virtual void onDeviceChangePopup();
     virtual void onMuteToggleButton();
-    virtual void onKill();
+    virtual void onKill(const Glib::VariantBase& parameter);
+    virtual void onDeviceComboBoxChanged();
 
 private:
     uint32_t mSinkIndex;
-
-    void clearMenu();
-    void buildMenu();
-
-    Gtk::Menu menu;
-
-    struct SinkMenuItem {
-      SinkMenuItem(SinkInputWidget *w, const char *label, uint32_t i, bool active) :
-      widget(w),
-      menuItem(label),
-      index(i) {
-        menuItem.set_active(active);
-        menuItem.set_draw_as_radio(true);
-        menuItem.signal_toggled().connect(sigc::mem_fun(*this, &SinkMenuItem::onToggle));
-      }
-
-      SinkInputWidget *widget;
-      Gtk::CheckMenuItem menuItem;
-      uint32_t index;
-      void onToggle();
-    };
-
-    std::map<uint32_t, SinkMenuItem*> sinkMenuItems;
 };
 
 #endif
